// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <cstring>
#include "xrm_scale_interface.h"
#include "xrm_utils.h"

/**
 * @brief Release the xrm resources which were allocated. Destroy the xrm
 * context.
 * 
 * @param scaler_xrm_ctx: Encoder XRM context
 * @return None
 */
void xrm_scale_release(XrmScaleContext* scaler_xrm_ctx) {
  if (!scaler_xrm_ctx->xrm_ctx) {
    return;
  }
  if (scaler_xrm_ctx->scaler_res_in_use) {
    /* Release the resource (still reserved) */
    xrmCuListReleaseV2(scaler_xrm_ctx->xrm_ctx, &scaler_xrm_ctx->scaler_cu_list_res);
    scaler_xrm_ctx->scaler_res_in_use = 0;
  }
  xrmDestroyContext(scaler_xrm_ctx->xrm_ctx);
}

/**
 * @brief Check & allocate the necessary xrm resources given the relevant 
 * scaler parameters
 * 
 * @param scaler_xrm_ctx Used to store information about resources this function
 * allocates so they can be safely released later
 * @param dev_index The device index
 * @param input_props Props used to calculate load
 * @param output_props Props used to calculate load (if the output affects the 
 * density such as scaler)
 * @param num_outputs Number of outputs in output_props
 * @return XRM_SUCCESS on success, XRM_ERROR on error or XRM_ERROR_CONNECT_FAIL
 * when unable to connect to daemon
 */
int32_t xrm_scale_reserve(XrmScaleContext* scaler_xrm_ctx, int dev_index, XrmInterfaceProperties* input_props, XrmInterfaceProperties* output_props, int num_outputs) {
  xrmCuPoolPropertyV2 sc_cu_pool_prop;
  memset(&sc_cu_pool_prop, 0, sizeof(sc_cu_pool_prop));

  scaler_xrm_ctx->dev_index = dev_index;

  /* Create XRM context */
  scaler_xrm_ctx->xrm_ctx = xrmCreateContext(XRM_API_VERSION_1);
  if (scaler_xrm_ctx->xrm_ctx == NULL) {
    return XRM_ERROR_CONNECT_FAIL;
  }

  /* Calculate scaler load based on scaler properties */
  int ret = xrm_utils_load_calc(scaler_xrm_ctx->xrm_ctx, input_props, output_props, num_outputs, "SCALER", &scaler_xrm_ctx->scaler_load, 0);
  if (scaler_xrm_ctx->scaler_load <= 0) {
    fprintf(stderr, "Scaler load calculation failed %d\n", ret);
    return XRM_ERROR;
  }

  xrm_utils_update_pool_props("SCALER", scaler_xrm_ctx->dev_index, scaler_xrm_ctx->scaler_load, &sc_cu_pool_prop, 0);
  ret                               = xrm_utils_reserve_resources(scaler_xrm_ctx->xrm_ctx, &sc_cu_pool_prop, &scaler_xrm_ctx->scaler_cu_list_res);
  scaler_xrm_ctx->scaler_res_in_use = ret == XRM_SUCCESS ? 1 : 0;
  scaler_xrm_ctx->dev_index         = scaler_xrm_ctx->scaler_cu_list_res.cuResources[0].deviceId;
  return ret;
}
